@@ -19,25 +19,27 @@ extern "C" {
  * xsltCompMatch:
  *
  * Data structure used for the implementation of patterns.
- * It is kept private (in pattern.c)
+ * It is kept private (in pattern.c).
  */
 typedef struct _xsltCompMatch xsltCompMatch;
 typedef xsltCompMatch *xsltCompMatchPtr;
 
 /*
- * Pattern related interfaces
+ * Pattern related interfaces.
  */
 
 xsltCompMatchPtr xsltCompilePattern	(const xmlChar *pattern,
 					 xmlDocPtr doc,
-					 xmlNodePtr node);
+					 xmlNodePtr node,
+					 xsltStylesheetPtr style,
+					 xsltTransformContextPtr runtime);
 void		 xsltFreeCompMatchList	(xsltCompMatchPtr comp);
 int		 xsltTestCompMatchList	(xsltTransformContextPtr ctxt,
 					 xmlNodePtr node,
 					 xsltCompMatchPtr comp);
 
 /*
- * Template related interfaces
+ * Template related interfaces.
  */
 int		xsltAddTemplate		(xsltStylesheetPtr style,
 					 xsltTemplatePtr cur,
@@ -49,11 +51,13 @@ xsltTemplatePtr	xsltGetTemplate		(xsltTransformContextPtr ctxt,
 void		xsltFreeTemplateHashes	(xsltStylesheetPtr style);
 void		xsltCleanupTemplates	(xsltStylesheetPtr style);
 
+#if 0
 int		xsltMatchPattern	(xsltTransformContextPtr ctxt,
 					 xmlNodePtr node,
 					 const xmlChar *pattern,
 					 xmlDocPtr ctxtdoc,
 					 xmlNodePtr ctxtnode);
+#endif
 #ifdef __cplusplus
 }
 #endif
