@@ -14,6 +14,9 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -49,7 +52,7 @@
 
 #include <libexslt/exsltconfig.h>
 
-#ifdef WIN32
+#if defined(WIN32) && !defined (__CYGWIN__)
 #ifdef _MSC_VER
 #include <winsock2.h>
 #pragma comment(lib, "ws2_32.lib")
@@ -112,9 +115,33 @@ static int errorno = 0;
  * Internal timing routines to remove the necessity to have unix-specific
  * function calls
  */
+#ifndef HAVE_GETTIMEOFDAY 
+#ifdef HAVE_SYS_TIMEB_H
+#ifdef HAVE_SYS_TIME_H
+#ifdef HAVE_FTIME
+
+int
+my_gettimeofday(struct timeval *tvp, void *tzp)
+{
+	struct timeb timebuffer;
+
+	ftime(&timebuffer);
+	if (tvp) {
+		tvp->tv_sec = timebuffer.time;
+		tvp->tv_usec = timebuffer.millitm * 1000L;
+	}
+	return (0);
+}
+#define HAVE_GETTIMEOFDAY 1
+#define gettimeofday my_gettimeofday
+
+#endif /* HAVE_FTIME */
+#endif /* HAVE_SYS_TIME_H */
+#endif /* HAVE_SYS_TIMEB_H */
+#endif /* !HAVE_GETTIMEOFDAY */
 
 #if defined(HAVE_GETTIMEOFDAY)
-static struct timeval begin, end;
+static struct timeval begin, endtime;
 /*
  * startTimer: call where you want to start timing
  */
@@ -132,10 +159,10 @@ static void endTimer(const char *format, ...)
     long msec;
     va_list ap;
 
-    gettimeofday(&end, NULL);
-    msec = end.tv_sec - begin.tv_sec;
+    gettimeofday(&endtime, NULL);
+    msec = endtime.tv_sec - begin.tv_sec;
     msec *= 1000;
-    msec += (end.tv_usec - begin.tv_usec) / 1000;
+    msec += (endtime.tv_usec - begin.tv_usec) / 1000;
 
 #ifndef HAVE_STDARG_H
 #error "endTimer required stdarg functions"
@@ -152,8 +179,11 @@ static void endTimer(const char *format, ...)
  * This is obviously less accurate, but there's little we can do about
  * that.
  */
+#ifndef CLOCKS_PER_SEC
+#define CLOCKS_PER_SEC 100
+#endif
 
-clock_t begin, end;
+clock_t begin, endtime;
 static void startTimer(void)
 {
     begin=clock();
@@ -163,8 +193,8 @@ static void endTimer(char *format, ...)
     long msec;
     va_list ap;
 
-    end=clock();
-    msec = ((end-begin) * 1000) / CLOCKS_PER_SEC;
+    endtime=clock();
+    msec = ((endtime-begin) * 1000) / CLOCKS_PER_SEC;
 
 #ifndef HAVE_STDARG_H
 #error "endTimer required stdarg functions"
@@ -205,6 +235,7 @@ static void endTimer(char *format, ...)
 static void
 xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
     xmlDocPtr res;
+    xsltTransformContextPtr ctxt;
 
 #ifdef LIBXML_XINCLUDE_ENABLED
     if (xinclude) {
@@ -239,11 +270,19 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 		    doc = xmlParseFile(filename);
 	    }
 	}
+	ctxt = xsltNewTransformContext(cur, doc);
+	if (ctxt == NULL)
+	    return;
 	if (profile) {
-	    res = xsltProfileStylesheet(cur, doc, params, stderr);
+	    res = xsltApplyStylesheetUser(cur, doc, params, NULL,
+		                          stderr, ctxt);
 	} else {
-	    res = xsltApplyStylesheet(cur, doc, params);
+	    res = xsltApplyStylesheetUser(cur, doc, params, NULL,
+		                          NULL, ctxt);
 	}
+	if (ctxt->state == XSLT_STATE_ERROR)
+	    errorno = 9;
+	xsltFreeTransformContext(ctxt);
 	if (timing) {
 	    if (repeat)
 		endTimer("Applying stylesheet %d times", repeat);
@@ -292,7 +331,21 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 
 	xmlFreeDoc(res);
     } else {
-	xsltRunStylesheet(cur, doc, params, output, NULL, NULL);
+	int ret;
+
+	ctxt = xsltNewTransformContext(cur, doc);
+	if (ctxt == NULL)
+	    return;
+	if (profile) {
+	    ret = xsltRunStylesheetUser(cur, doc, params, output,
+		                        NULL, NULL, stderr, ctxt);
+	} else {
+	    ret = xsltRunStylesheetUser(cur, doc, params, output,
+		                        NULL, NULL, NULL, ctxt);
+	}
+	if (ctxt->state == XSLT_STATE_ERROR)
+	    errorno = 9;
+	xsltFreeTransformContext(ctxt);
 	if (timing)
 	    endTimer("Running stylesheet and saving result");
 	xmlFreeDoc(doc);
@@ -318,7 +371,7 @@ static void usage(const char *name) {
     printf("\t--docbook: the input document is SGML docbook\n");
 #endif
     printf("\t--param name value : pass a (parameter,value) pair\n");
-    printf("\t       value is an XPath expression.");
+    printf("\t       value is an XPath expression.\n");
     printf("\t       string values must be quoted like \"'string'\"\n or");
     printf("\t       use stringparam to avoid it\n");
     printf("\t--stringparam name value : pass a (parameter,string value) pair\n");
@@ -332,6 +385,8 @@ static void usage(const char *name) {
     printf("\t--xinclude : do XInclude processing on document intput\n");
 #endif
     printf("\t--profile or --norman : dump profiling informations \n");
+    printf("\nProject libxslt home page: http://xmlsoft.org/XSLT/\n");
+    printf("To report bugs and get help: http://xmlsoft.org/XSLT/bugs.html\n");
 }
 
 int
@@ -371,7 +426,7 @@ main(int argc, char **argv)
                    (!strcmp(argv[i], "-output")) ||
                    (!strcmp(argv[i], "--output"))) {
             i++;
-            output = argv[i++];
+            output = argv[i];
         } else if ((!strcmp(argv[i], "-V")) ||
                    (!strcmp(argv[i], "-version")) ||
                    (!strcmp(argv[i], "--version"))) {
@@ -525,6 +580,11 @@ main(int argc, char **argv)
             i += 2;
             continue;
         }
+        if ((!strcmp(argv[i], "-stringparam")) ||
+            (!strcmp(argv[i], "--stringparam"))) {
+            i += 2;
+            continue;
+        }
         if ((argv[i][0] != '-') || (strcmp(argv[i], "-") == 0)) {
             if (timing)
                 startTimer();
@@ -600,7 +660,9 @@ main(int argc, char **argv)
 done:
     xsltCleanupGlobals();
     xmlCleanupParser();
+#if 0
     xmlMemoryDump();
+#endif
     return(errorno);
 }
 
